package com.fr.plugin.ztzzSSO.filter;

import com.fr.base.TemplateUtils;
import com.fr.decision.fun.impl.AbstractEmbedRequestFilterProvider;
import com.fr.decision.webservice.bean.authentication.OriginUrlResponseBean;
import com.fr.decision.webservice.utils.DecisionStatusService;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.ztzzSSO.config.PluginSimpleConfig;
import com.fr.plugin.ztzzSSO.utils.FRUtils;
import com.fr.plugin.ztzzSSO.utils.HttpUtils;
import com.fr.plugin.ztzzSSO.utils.ResponseUtils;
import com.fr.plugin.ztzzSSO.utils.Utils;
import com.fr.stable.StringUtils;
import com.fr.stable.fun.Authorize;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

@Authorize(callSignKey = "com.fr.plugin.ztzzSSO")
public class SSOFilter extends AbstractEmbedRequestFilterProvider {

    @Override
    public void filter(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        if(!PluginContexts.currentContext().isAvailable()) {
            ResponseUtils.failedResponse(httpServletResponse,"插件已过期，请联系管理员！");
            return ;
        }

        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        try {
            if(isRelease(httpServletRequest,httpServletResponse,psc)){
                return ;
            }
        } catch (Exception e) {
            ResponseUtils.failedResponse(httpServletResponse,"放行异常！");
            return;
        }

        //访问路径
        String url = FRUtils.getAllUrl(httpServletRequest,psc.getFrDomain());
        //是否从一体化进入
        String fromSSO = httpServletRequest.getParameter("fromSSO");

        //从一体化进入每次都重新请求code
        if(Utils.isNotNullStr(fromSSO) && "1".equals(fromSSO)){
            FRUtils.FRLogInfo("一体化跳转:"+url);
            String redirectUrl = url.substring(0,url.indexOf("?fromSSO"));
            FRUtils.FRLogInfo("一体化跳转,截取后url:"+url);

            redirect(httpServletRequest,httpServletResponse,redirectUrl);
            return ;
        }

        //是否从模板访问到此
        if(url.contains("decision/login")){
            String origin = httpServletRequest.getParameter("origin");

            if(Utils.isNullStr(origin)){
                return ;
            }
            String originurl = "";
            try {
                originurl = getOriginUrlResponse(origin);
            } catch (Exception e) {
                ResponseUtils.failedResponse(httpServletResponse,"获取来源链接失败!");
            }

            if(!(originurl.contains("viewlet=") && (originurl.contains(".cpt") || originurl.contains(".frm"))) || originurl.contains("design")){
                return ;
            }

            originurl = psc.getFrDomain()+originurl;
            redirect(httpServletRequest,httpServletResponse,originurl);
            return ;
        }

        //获取code
        String code = httpServletRequest.getParameter("code");
        String msg = httpServletRequest.getParameter("msg");

        //跳转 没有code
        if(Utils.isNullStr(code) && Utils.isNullStr(msg)){
            redirect(httpServletRequest,httpServletResponse,url);
            return ;
        }

        //获取token
        JSONObject tokenResultJson = getToken(code);
        String token = tokenResultJson.getString("access_token");

        if(Utils.isNullStr(token)){
            ResponseUtils.failedResponse(httpServletResponse,"获取token异常:");
        }

        //获取用户信息
        String uid = tokenResultJson.getString("uid");

        String loginName = getUsername(token,uid);

        if(Utils.isNullStr(loginName)){
            FRUtils.FRLogInfo("获取用户信息异常:");
            ResponseUtils.failedResponse(httpServletResponse,"获取用户信息异常:");

            return;
        }

        if(url.contains("code=")){
            url = url.contains("&code") ? url.substring(0,url.indexOf("&code")) :url.substring(0,url.indexOf("?code"));
        }

        FRUtils.FRLogInfo("登陆成功-url:"+url);

        FRUtils.login(httpServletRequest,httpServletResponse,loginName,url);

    }

    private String getUsername(String token,String uid){
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String clientId = psc.getAppId();

        String userInfoUrl = psc.getUserInfoUrl();
        userInfoUrl+="?access_token="+token+"&client_id="+clientId+"&uid="+uid;

        String userInfoResult = HttpUtils.get(userInfoUrl,null,null);

        if(Utils.isNullStr(userInfoResult)){
            FRUtils.FRLogInfo("获取用户信息异常");
            return "";
        }

        JSONObject userJsonObject = new JSONObject(userInfoResult);

        String loginName = userJsonObject.getString("loginName");

        return loginName;
    }

    private void redirect(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse,String redirectUrl){
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String clientId = psc.getAppId();

        String authUrl = psc.getCodeUrl();

        try {
            redirectUrl= URLEncoder.encode(redirectUrl,"utf-8");
        } catch (UnsupportedEncodingException e) {
            return;
        }

        authUrl+= "?redirect_uri="+redirectUrl+"&state=123&client_id="+clientId+"&response_type=code";
        try {
            httpServletResponse.sendRedirect(authUrl);
        } catch (IOException e) {
            FRUtils.FRLogInfo("跳转异常:"+e.getMessage());
        }
    }

    private JSONObject getToken(String code){
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String clientId = psc.getAppId();
        String clientSecret= psc.getHost();
        //获取token
        String tokenUrl = psc.getTokenUrl()+"?client_id="+clientId+"&client_secret="+clientSecret+"&code="+code+"&grant_type=authorization_code";
        JSONObject tokenParam = new JSONObject();
        tokenParam.put("client_id",clientId);
        tokenParam.put("client_secret",clientSecret);
        tokenParam.put("code",code);
        tokenParam.put("grant_type","authorization_code");

        String tokenParamStr = tokenParam.toString();

        String tokenResult = HttpUtils.HttpPostJson(tokenUrl,tokenParamStr,null);

        if(Utils.isNullStr(tokenResult)){
            return new JSONObject();
        }

        JSONObject tokenResultJson = new JSONObject(tokenResult);

        return tokenResultJson;
    }

    private static boolean isRelease(HttpServletRequest req,HttpServletResponse res,PluginSimpleConfig psc) throws Exception {
        //请求url,ngix映射有问题，手动将ip换为域名
        String url = FRUtils.getAllUrl(req,psc.getFrDomain());

        //远程设计
        String reft = req.getParameter("ref_t");

        //是否需要拦截 系统首页、模板链接
        boolean isViewlet = (url.contains("viewlet=") && (url.contains(".cpt") || url.contains(".frm"))) ;

        //是否登陆
        boolean isLogin = FRUtils.isLogin(req);

        String fromSSO = req.getParameter("fromSSO");
        //远程设计放行 或者 (已经登录 || 不是系统首页) && 不来自认证中心 && 不是登录页放行
        return "design".equals(reft) || (isLogin && !(Utils.isNotNullStr(fromSSO) && "1".equals(fromSSO))) || (!url.equals(psc.getFrDomain()+"/webroot/decision") && !url.contains("decision/login") && !((Utils.isNotNullStr(fromSSO) && "1".equals(fromSSO)) || url.contains("code=") || url.contains("url/mobile")));
    }

    /**
     *
     * @param originKey
     * @return
     * @throws Exception
     */
    private String getOriginUrlResponse(String originKey) throws Exception {
        if (StringUtils.isNotEmpty(originKey)) {
            OriginUrlResponseBean originUrlResponseBean = (OriginUrlResponseBean) DecisionStatusService.originUrlStatusService().get(originKey);
            DecisionStatusService.originUrlStatusService().delete(originKey);
            if (originUrlResponseBean != null) {
                return originUrlResponseBean.getOriginUrl();
            }
        }

        return new OriginUrlResponseBean(TemplateUtils.render("${fineServletURL}")).getOriginUrl();
    }
}