package com.fr.plugin.ztzzSSO.serverDS;

import com.fr.base.TableData;
import com.fr.data.AbstractDataModel;
import com.fr.general.data.TableDataException;
import com.fr.general.http.HttpClient;
import com.fr.general.http.HttpToolbox;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.ztzzSSO.config.PluginSimpleConfig;
import com.fr.plugin.ztzzSSO.utils.FRUtils;
import com.fr.plugin.ztzzSSO.utils.HttpUtils;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class UserHttpDataModel extends AbstractDataModel {
    private static String[] COLUMN_NAMES = {"Mobile_Phone", "Email", "Gender","Name","Main_Org_Id",
    "Account_Number","Main_Org_Id__ref","Gender__ref"};
    private int rowCount = TableData.RESULT_ALL;
    private DatasetData datas = new DatasetData();

    public UserHttpDataModel(int count) {
        this.rowCount = count;
        if (this.rowCount == 0) {
            return;
        }
        queryData();
    }

    //获取列的数量
    @Override
    public int getColumnCount() throws TableDataException {
        return COLUMN_NAMES.length;
    }

    //获取列的名称
    @Override
    public String getColumnName(int i) throws TableDataException {
        return COLUMN_NAMES[i];
    }

    //是否还有行
    @Override
    public boolean hasRow(int rowIndex) throws TableDataException {
        int count = getRowCount();
        return rowIndex < count;
    }

    //获取行数量
    @Override
    public int getRowCount() throws TableDataException {
        if (this.datas == null) {
            return 0;
        }
        List<List<Object>> values = this.datas.getValues();
        if (values == null) {
            return 0;
        }
        int count = values.size();
        return count;
    }

    //获取数据
    @Override
    public Object getValueAt(int rowIndex, int columnIndex){
        if (this.datas == null) {
            return "";
        }
        List<List<Object>> values = this.datas.getValues();
        if ((values == null) || (values.size() <= rowIndex)) {
            return "";
        }
        List<Object> rowValues = values.get(rowIndex);
        if ((rowValues == null) || (rowValues.size() <= columnIndex)) {
            return "";
        }
        return rowValues.get(columnIndex);
    }

    //释放
    @Override
    public void release() {
        this.datas = null;
    }

    /**
     * 查询数据
     */
    private void queryData() {
        try {
            String token = getToken();
            createDatas(0,token);
        } catch (Exception e) {
            FRUtils.FRLogInfo("数据集获取用户信息,请求出错:"+e.getMessage());
            throw new NullPointerException("数据集获取用户信息为空," + e.getMessage());
        }
    }

    /**
     * 获取token
     * @return
     */
    private String getToken(){
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String tokenUrl = psc.getTokenUrl2();
        String username = psc.getUsername();
        String psd = psc.getPsd();

        Map<String,String> header = new HashMap<String,String>();
        header.put("Content-Type","application/json");
        header.put("X-MDM-UNIFIED-RESPONSE","true");

        JSONObject params = new JSONObject();
//        Map<String,String> params = new HashMap<String,String>();
        params.put("username",username);
        params.put("password",psd);

        String result = "";

        try {
            result = HttpUtils.HttpPostJson(tokenUrl,params.toString(),header);
        } catch (Exception e) {
            return "";
        }

        JSONObject resultJson = new JSONObject(result);

        return resultJson.getString("token");
    }

    /**
     * 获取组织数据
     * @param pageNum
     * @param token
     * @return
     */
    private JSONObject getUser(int pageNum,String token){
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String orgUrl = psc.getUserUrl()+"?pageNum="+pageNum+"&pageSize=1000";
        String result = httpRequest(orgUrl,"POST",null,token);

        return new JSONObject(result);
    }

    public static String httpRequest(String requestUrl, String requestMethod,
                                     String outputStr, String tokenid) {
        StringBuffer buffer = null;
        HttpURLConnection conn = null;
        InputStream is = null;
        InputStreamReader isr = null;
        try {
            URL url = new URL(requestUrl);
            conn = (HttpURLConnection) url.openConnection();
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setRequestMethod(requestMethod);
            conn.setRequestProperty("Content-Type", "application/json");
            conn.setRequestProperty("Authorization", tokenid);
            conn.setRequestProperty("X-MDM-UNIFIED-RESPONSE", "true");
            conn.connect();
            // 往服务器端写内容 也就是发起http请求需要带的参数
            if (null != outputStr) {
                OutputStream os = conn.getOutputStream();
                os.write(outputStr.getBytes("utf-8"));
                os.close();
            }

            // 读取服务器端返回的内容
            is = conn.getInputStream();
            isr = new InputStreamReader(is, "utf-8");
            BufferedReader br = new BufferedReader(isr);
            buffer = new StringBuffer();
            String line = null;
            while ((line = br.readLine()) != null) {
                buffer.append(line);
            }
            System.out.println(buffer);
        } catch (Exception e) {
            e.printStackTrace();
        }finally{
            if(null != conn){
                conn.disconnect();
            }
            if(null != is){
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if(null != isr){
                try {
                    isr.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return buffer.toString();
    }

    private void createDatas(int pageNum,String token) {
        JSONObject json = getUser(pageNum,token);
        JSONArray jsonArray = json.getJSONArray("content");
        addRowDatas(jsonArray);

        boolean last = json.getBoolean("last");
        if(last){
            return;
        }

        pageNum = pageNum+1;

        createDatas(pageNum,token);
    }

    /**
     * 添加行数据
     * @param jsonArray
     */
    private void addRowDatas(JSONArray jsonArray) {
        if(jsonArray == null || jsonArray.size() <= 0){
            return ;
        }

        for(int i =0;i< jsonArray.size();i++){
            JSONObject org = jsonArray.getJSONObject(i);
            String  Mobile_Phone = org.getString("Mobile_Phone");
            String  Email = org.getString("Email");
            String  Gender = org.getString("Gender");
            String  Name = org.getString("Name");
            String  Main_Org_Id = org.getString("Main_Org_Id");
            String  Account_Number = org.getString("Account_Number");
            String  Main_Org_Id__ref = org.getString("Main_Org_Id__ref");
            String  Gender__ref = org.getString("Gender__ref");

            List<Object> rowDatas = new ArrayList<Object>();

            rowDatas.add(Mobile_Phone);
            rowDatas.add(Email);
            rowDatas.add(Gender);
            rowDatas.add(Name);
            rowDatas.add(Main_Org_Id);
            rowDatas.add(Account_Number);
            rowDatas.add(Main_Org_Id__ref);
            rowDatas.add(Gender__ref);


            List<List<Object>> values = this.datas.getValues();
            values.add(rowDatas);

        }
    }
}
